/***************************************************************************
 *   Copyright (C) 2007-2008 by Riccardo Iaconelli <riccardo@kde.org>      *
 *   Copyright (C) 2007-2008 by Sebastian Kuegler <sebas@kde.org>          *
 *   Copyright (C) 2007 by Luka Renko <lure@kubuntu.org>                   *
 *   Copyright (C) 2008 by Thomas Gillespie <tomjamesgillespie@gmail.com>  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "powersave.h"

#include <QApplication>
#include <QPainter>
#include <QStyleOptionGraphicsItem>
#include <QFont>
#include <QGraphicsSceneHoverEvent>

#include <KDebug>
#include <KIcon>
#include <KLocalizedString>
#include <KSharedConfig>
#include <KDialog>
#include <KColorScheme>
#include <KConfigDialog>
#include <KGlobalSettings>
#include <KPassivePopup>

#include <plasma/svg.h>
#include <plasma/theme.h>
#include <plasma/animator.h>

Powersave::Powersave(QObject *parent, const QVariantList &args)
    : Plasma::Applet(parent, args),
      m_batteryStyle(0),
      m_theme(0),
      displayManager(new KDisplayManager()),
      eventNotifier(Solid::Control::PowerManager::notifier()),
      m_animId(-1),
      m_alpha(1),
      m_fadeIn(true),
      m_acAnimId(-1),
      m_acAlpha(1),
      m_acFadeIn(false),
      m_batteryAnimId(-1),
      m_batteryAlpha(1),
      m_batteryFadeIn(true),
      m_isHovered(false),
      m_numOfBattery(0),
      configOpen(false),
      currentBrightness(Solid::Control::PowerManager::brightness()),
      pollTimer(new QTimer())
{
    kDebug() << "Loading applet powersave";
    setAcceptsHoverEvents(true);
    setHasConfigurationInterface(true);
    resize(128, 128);
    //setAspectRatioMode(Plasma::ConstrainedSquare );
    m_textRect = QRect();
    
    //Code idea borrowed from kSnapshot
    m_grabber = new QWidget(0, Qt::X11BypassWindowManagerHint);
    m_grabber->move(-1000, -1000);
    m_grabber->setMouseTracking(true);
    m_grabber->installEventFilter( this );
}

void Powersave::init()
{
    KConfigGroup cg = config();

    using namespace Solid::Control::PowerManager;
    configLockScreen = cg.readEntry("lockScreen", true);
    dimOnIdle = cg.readEntry("dimOnIdle", true);
    acSchemeConfig.brightness = cg.readEntry("acSchemeConfig.brightness", 50);
    acSchemeConfig.offDisplayWhenIdle = cg.readEntry("acSchemeConfig.offDisplayWhenIdle", true);
    acSchemeConfig.displayIdleTime = cg.readEntry("acSchemeConfig.displayIdleTime", 60);
    acSchemeConfig.idleTime = cg.readEntry("acSchemeConfig.idleTime", 50);
    acSchemeConfig.idleAction = (IdleAction) cg.readEntry("acSchemeConfig.idleAction", 0);
    acSchemeConfig.cpuPolicy = (CpuFreqPolicy) cg.readEntry("acSchemeConfig.cpuPolicy", (int) OnDemand);
    acSchemeConfig.lidCloseAction = (IdleAction) cg.readEntry("acSchemeConfig.lidCloseAction", 4);
    
    batSchemeConfig.brightness = cg.readEntry("batSchemeConfig.brightness", 50);
    batSchemeConfig.offDisplayWhenIdle = cg.readEntry("batSchemeConfig.offDisplayWhenIdle", true);
    batSchemeConfig.displayIdleTime = cg.readEntry("batSchemeConfig.displayIdleTime", 10);
    batSchemeConfig.idleTime = cg.readEntry("batSchemeConfig.idleTime", 50);
    batSchemeConfig.idleAction = (IdleAction) cg.readEntry("batSchemeConfig.idleAction", 0);
    batSchemeConfig.cpuPolicy =  (CpuFreqPolicy) cg.readEntry("batSchemeConfig.cpuPolicy", (int) Solid::Control::PowerManager::Powersave);
    batSchemeConfig.lidCloseAction = (IdleAction) cg.readEntry("batSchemeConfig.lidCloseAction", 0);

    batLowAction = (IdleAction) cg.readEntry("batLowAction", 0);

    m_showBatteryString = cg.readEntry("showPowersaveString", false);
    m_showMultipleBatteries = cg.readEntry("showMultipleBatteries", true);

    QString svgFile = QString();
    if (cg.readEntry("style", 0) == 0) {
        m_batteryStyle = OxygenBattery;
        svgFile = "widgets/battery-oxygen";
    } else {
        m_batteryStyle = ClassicBattery;
        svgFile = "widgets/battery";
    }
    m_theme = new Plasma::Svg(this);
    m_theme->setImagePath(svgFile);
    m_theme->setContainsMultipleImages(false);

    m_theme->resize(contentsRect().size());
    m_font = QApplication::font();
    m_font.setWeight(QFont::Bold);

    m_boxAlpha = 128;
    m_boxHoverAlpha = 192;

    readColors();
    connect(Plasma::Theme::defaultTheme(), SIGNAL(themeChanged()), SLOT(readColors()));

    const QStringList& battery_sources = dataEngine("powermanagement")->query(I18N_NOOP("Battery"))[I18N_NOOP("sources")].toStringList();

    //connect sources
    connectSources();

    foreach (const QString &battery_source, battery_sources) {
        kDebug() << "PowersaveSource:" << battery_source;
        dataUpdated(battery_source, dataEngine("powermanagement")->query(battery_source));
    }
    dataUpdated(I18N_NOOP("AC Adapter"), dataEngine("powermanagement")->query(I18N_NOOP("AC Adapter")));
    m_numOfBattery = battery_sources.size();
    kDebug() << battery_sources.size();

    //connect to dbus
    m_screenSaverIface = new OrgFreedesktopScreenSaverInterface("org.freedesktop.ScreenSaver", "/ScreenSaver", 
                                                                 QDBusConnection::sessionBus(), this);
    //build the popup menu
    buildMenu();

    //Connect events
    connect(eventNotifier, SIGNAL(acAdapterStateChanged(int)), this, SLOT(acAdapterStateChanged(int)));
    connect(eventNotifier, SIGNAL(batteryStateChanged(int)), this, SLOT(batteryStateChanged(int)));
    connect(eventNotifier, SIGNAL(buttonPressed(int)), this, SLOT(buttonPressed(int)));

    //setup idle timer, can remove it if we can get some kind of idle time signal from dbus?
    pollTimer->setInterval(700);
    connect(pollTimer, SIGNAL(timeout()), this, SLOT(poll()));
    connect(this, SIGNAL(configNeedsSaving()), this, SLOT(poll()));
    pollTimer->start();

    //Setup initial state
    acAdapterStateChanged(Solid::Control::PowerManager::acAdapterState());
}

void Powersave::constraintsEvent(Plasma::Constraints constraints)
{
    if (constraints & Plasma::FormFactorConstraint) {
        if (formFactor() == Plasma::Vertical) {
            setMaximumHeight(qMax(m_textRect.height(), contentsRect().width()));
            kDebug() << "Vertical FormFactor";
            // TODO: set background(true) on panel causes 0 height, so do not use it
        } else if (formFactor() == Plasma::Horizontal) {
            setMaximumWidth(qMax(m_textRect.width(), contentsRect().height()));
            kDebug() << "Horizontal FormFactor";
            // TODO: set background(true) on panel causes 0 height, so do not use it
        }
    }

    if (constraints & (Plasma::SizeConstraint | Plasma::FormFactorConstraint) && m_theme) {
        m_theme->resize(contentsRect().size().toSize());
        m_font.setPointSize(qMax(KGlobalSettings::smallestReadableFont().pointSize(),
                                 qRound(contentsRect().height() / 10)));
    }
}

///Builds the context menu of the systray icon
void Powersave::buildMenu()
{
    mainMenu = new KMenu();
    //Suspend
    actionSuspendToDisk = mainMenu->addAction(KIcon("drive-harddisk"), i18n("Suspend To Disk"), this, SLOT(suspendToDisk()));
    actionSuspendToRam = mainMenu->addAction(KIcon("media-flash"), i18n("Suspend To RAM"), this, SLOT(suspendToRam()));
    actionStandby = mainMenu->addAction(KIcon("media-flash"), i18n("Standby"), this, SLOT(standby()));
    mainMenu->addSeparator();
    //Frequency Policies Submenu
    QMenu * cpuSubMenu = mainMenu->addMenu(KIcon("cpu"), i18n("Set CPU Frequency Policy"));
    Solid::Control::PowerManager::CpuFreqPolicies policies = Solid::Control::PowerManager::supportedCpuFreqPolicies();
    Solid::Control::PowerManager::CpuFreqPolicy currentPolicy = Solid::Control::PowerManager::cpuFreqPolicy();
    if(policies | Solid::Control::PowerManager::Performance)
    {
        actionCpuPolicyPerformance = cpuSubMenu->addAction("Performance", this, SLOT(cpuPolicyClicked()));
        actionCpuPolicyPerformance->setProperty("CPUPolicy", QVariant(Solid::Control::PowerManager::Performance));
        actionCpuPolicyPerformance->setCheckable(true);
    }
    if(policies | Solid::Control::PowerManager::OnDemand)
    {
        actionCpuPolicyOnDemand = cpuSubMenu->addAction("Dynamic (aggressive)", this, SLOT(cpuPolicyClicked()));
        actionCpuPolicyOnDemand->setProperty("CPUPolicy", QVariant(Solid::Control::PowerManager::OnDemand));
        actionCpuPolicyOnDemand->setCheckable(true);
    }
    if(policies | Solid::Control::PowerManager::Conservative)
    {
        actionCpuPolicyConservative = cpuSubMenu->addAction("Dynamic (conservative)", this, SLOT(cpuPolicyClicked()));
        actionCpuPolicyConservative->setProperty("CPUPolicy", QVariant(Solid::Control::PowerManager::Conservative));
        actionCpuPolicyConservative->setCheckable(true);
    }
    if(policies | Solid::Control::PowerManager::Powersave)
    {
        actionCpuPolicyPowersave = cpuSubMenu->addAction("Powersave", this, SLOT(cpuPolicyClicked()));
        actionCpuPolicyPowersave->setProperty("CPUPolicy", QVariant(Solid::Control::PowerManager::Powersave));
        actionCpuPolicyPowersave->setCheckable(true);
    }
    if(currentPolicy == Solid::Control::PowerManager::Performance)
    {
        actionCpuPolicyPerformance->setChecked(true);
    }
    else if(currentPolicy == Solid::Control::PowerManager::OnDemand)
    {
        actionCpuPolicyOnDemand->setChecked(true);
    }
    else if(currentPolicy == Solid::Control::PowerManager::Conservative)
    {
        actionCpuPolicyConservative->setChecked(true);
    }
    else if(currentPolicy == Solid::Control::PowerManager::Powersave)
    {
        actionCpuPolicyPowersave->setChecked(true);
    }
    mainMenu->addSeparator();
    
    actionBrightnessSlider = new QSlider(Qt::Horizontal);
    actionBrightnessSlider->setValue(currentBrightness);
    QWidgetAction *actionBrightnessSliderWidget = new QWidgetAction(mainMenu);
    actionBrightnessSliderWidget->setDefaultWidget(actionBrightnessSlider);
    connect(actionBrightnessSlider, SIGNAL(valueChanged(int)), this, SLOT(configBrightnessChanged(int)));
    mainMenu->addAction(actionBrightnessSliderWidget);

    //Set the icon
    mainMenu->setIcon(KIcon("devices/battery"));
}

void Powersave::readConfigToUi()
{
    using namespace Solid::Control::PowerManager;
    ui.lockScreenOnResume->setChecked(configLockScreen);
    ui.dimDisplayOnIdle->setChecked(dimOnIdle);
    
    ui.PoweredBrightnessSlider->setValue(acSchemeConfig.brightness);
    ui.PoweredOffDisplayWhenIdle->setChecked(acSchemeConfig.offDisplayWhenIdle);
    ui.PoweredDisplayIdleTime->setValue(acSchemeConfig.displayIdleTime);
    ui.PoweredIdleTime->setValue(acSchemeConfig.idleTime);
    ui.PoweredIdleCombo->setCurrentIndex(ui.PoweredIdleCombo->findData(acSchemeConfig.idleAction));
    ui.PoweredFreqCombo->setCurrentIndex(ui.PoweredFreqCombo->findData(acSchemeConfig.cpuPolicy));
    ui.BatteryBrightnessSlider->setValue(batSchemeConfig.brightness);
    ui.BatteryOffDisplayWhenIdle->setChecked(batSchemeConfig.offDisplayWhenIdle);
    ui.BatteryDisplayIdleTime->setValue(batSchemeConfig.displayIdleTime);
    ui.BatteryIdleTime->setValue(batSchemeConfig.idleTime);
    ui.BatteryIdleCombo->setCurrentIndex(ui.BatteryIdleCombo->findData(batSchemeConfig.idleAction));
    ui.BatteryFreqCombo->setCurrentIndex(ui.BatteryFreqCombo->findData(batSchemeConfig.cpuPolicy));

    ui.BatteryCriticalCombo->setCurrentIndex(ui.BatteryCriticalCombo->findData(batLowAction));
    
    switch(batSchemeConfig.lidCloseAction)
    {
        case S2Disk:
            ui.laptopClosedBatteryHibernate->setChecked(true);
            break;
        case S2Ram:
            ui.laptopClosedBatterySuspend->setChecked(true);
            break;
        case Shutdown:
            ui.laptopClosedBatteryShutdown->setChecked(true);
            break;
        case Lock:
            ui.laptopClosedBatteryBlank->setChecked(true);
            break;
        case None:
        default:
            ui.laptopClosedBatteryNone->setChecked(true);
            break;
    }
    switch(acSchemeConfig.lidCloseAction)
    {
        case S2Disk:
            ui.laptopClosedACHibernate->setChecked(true);
            break;
        case S2Ram:
            ui.laptopClosedACSuspend->setChecked(true);
            break;
        case Shutdown:
            ui.laptopClosedACShutdown->setChecked(true);
            break;
        case Lock:
            ui.laptopClosedACBlank->setChecked(true);
            break;
        case None:
        default:
            ui.laptopClosedACNone->setChecked(true);
            break;
    }
}

void Powersave::writeConfigFromUi()
{
    KConfigGroup cg = config();

    using namespace Solid::Control::PowerManager;
    if(configLockScreen != (ui.lockScreenOnResume->checkState() == Qt::Checked))
    {
        configLockScreen = !configLockScreen;
        cg.writeEntry("lockScreen", configLockScreen);
    }
    if(dimOnIdle != (ui.dimDisplayOnIdle->checkState() == Qt::Checked))
    {
        dimOnIdle = !dimOnIdle;
        cg.writeEntry("dimOnIdle", ui.dimDisplayOnIdle->checkState() == Qt::Checked);
    }
    if(acSchemeConfig.brightness != ui.PoweredBrightnessSlider->value())
    {
        acSchemeConfig.brightness = ui.PoweredBrightnessSlider->value();
        cg.writeEntry("acSchemeConfig.brightness", acSchemeConfig.brightness);
    }
    if(acSchemeConfig.offDisplayWhenIdle != (ui.PoweredOffDisplayWhenIdle->checkState() == Qt::Checked))
    {
        acSchemeConfig.offDisplayWhenIdle = !acSchemeConfig.offDisplayWhenIdle;
        cg.writeEntry("acSchemeConfig.offDisplayWhenIdle", ui.PoweredOffDisplayWhenIdle->checkState() == Qt::Checked);
    }
    if(acSchemeConfig.displayIdleTime != ui.PoweredDisplayIdleTime->value())
    {
        acSchemeConfig.displayIdleTime = ui.PoweredDisplayIdleTime->value();
        cg.writeEntry("acSchemeConfig.displayIdleTime", ui.PoweredDisplayIdleTime->value());
    }
    if(acSchemeConfig.idleTime != ui.PoweredIdleTime->value())
    {
        acSchemeConfig.idleTime = ui.PoweredIdleTime->value();
        cg.writeEntry("acSchemeConfig.idleTime", ui.PoweredIdleTime->value());
    }
    if(acSchemeConfig.idleAction !=  ui.PoweredIdleCombo->itemData(ui.PoweredIdleCombo->currentIndex()).toInt())
    {
        acSchemeConfig.idleAction = (IdleAction) ui.PoweredIdleCombo->itemData(ui.PoweredIdleCombo->currentIndex()).toInt();
        cg.writeEntry("acSchemeConfig.idleAction", ui.PoweredIdleCombo->itemData(ui.PoweredIdleCombo->currentIndex()).toInt());
    }
    if(acSchemeConfig.cpuPolicy != ui.PoweredFreqCombo->itemData(ui.PoweredFreqCombo->currentIndex()).toInt())
    {
        acSchemeConfig.cpuPolicy = (CpuFreqPolicy) ui.PoweredFreqCombo->itemData(ui.PoweredFreqCombo->currentIndex()).toInt();
        cg.writeEntry("acSchemeConfig.cpuPolicy", ui.PoweredFreqCombo->itemData(ui.PoweredFreqCombo->currentIndex()).toInt());
    }
    if(batSchemeConfig.brightness != ui.BatteryBrightnessSlider->value())
    {
        batSchemeConfig.brightness = ui.BatteryBrightnessSlider->value();
        cg.writeEntry("batSchemeConfig.brightness", ui.BatteryBrightnessSlider->value());
    }
    if(batSchemeConfig.offDisplayWhenIdle != (ui.BatteryOffDisplayWhenIdle->checkState() == Qt::Checked))
    {
        batSchemeConfig.offDisplayWhenIdle = !batSchemeConfig.offDisplayWhenIdle;
        cg.writeEntry("batSchemeConfig.offDisplayWhenIdle", ui.BatteryOffDisplayWhenIdle->checkState() == Qt::Checked);
    }
    if(batSchemeConfig.displayIdleTime != ui.BatteryDisplayIdleTime->value())
    {
        batSchemeConfig.displayIdleTime = ui.BatteryDisplayIdleTime->value();
        cg.writeEntry("batSchemeConfig.displayIdleTime", ui.BatteryDisplayIdleTime->value());
    }
    if(batSchemeConfig.idleTime != ui.BatteryIdleTime->value())
    {
        batSchemeConfig.idleTime = ui.BatteryIdleTime->value();
        cg.writeEntry("batSchemeConfig.idleTime", ui.BatteryIdleTime->value());
    }
    if(batSchemeConfig.idleAction != ui.BatteryIdleCombo->itemData(ui.BatteryIdleCombo->currentIndex()).toInt())
    {
        batSchemeConfig.idleAction = (IdleAction) ui.BatteryIdleCombo->itemData(ui.BatteryIdleCombo->currentIndex()).toInt();
        cg.writeEntry("batSchemeConfig.idleAction", ui.BatteryIdleCombo->itemData(ui.BatteryIdleCombo->currentIndex()).toInt());
    }
    if(batLowAction != ui.BatteryCriticalCombo->itemData(ui.BatteryCriticalCombo->currentIndex()).toInt())
    {
        batLowAction = (IdleAction) ui.BatteryCriticalCombo->itemData(ui.BatteryCriticalCombo->currentIndex()).toInt();
        cg.writeEntry("batLowAction", ui.BatteryCriticalCombo->itemData(ui.BatteryCriticalCombo->currentIndex()).toInt());
    }
    if(batSchemeConfig.cpuPolicy != ui.BatteryFreqCombo->itemData(ui.BatteryFreqCombo->currentIndex()).toInt())
    {
        batSchemeConfig.cpuPolicy =  (CpuFreqPolicy) ui.BatteryFreqCombo->itemData(ui.BatteryFreqCombo->currentIndex()).toInt();
        cg.writeEntry("batSchemeConfig.cpuPolicy", ui.BatteryFreqCombo->itemData(ui.BatteryFreqCombo->currentIndex()).toInt());
    }
    if(ui.laptopClosedBatteryNone->isChecked())
    {
        batSchemeConfig.lidCloseAction = None;
        cg.writeEntry("batSchemeConfig.lidCloseAction", (int) None);
    }
    else if(ui.laptopClosedBatteryHibernate->isChecked())
    {
        batSchemeConfig.lidCloseAction = S2Disk;
        cg.writeEntry("batSchemeConfig.lidCloseAction", (int) S2Disk);
    }
    else if(ui.laptopClosedBatterySuspend->isChecked())
    {
        batSchemeConfig.lidCloseAction = S2Ram;
        cg.writeEntry("batSchemeConfig.lidCloseAction", (int) S2Ram);
    }
    else if(ui.laptopClosedBatteryShutdown->isChecked())
    {
        batSchemeConfig.lidCloseAction = Shutdown;
        cg.writeEntry("batSchemeConfig.lidCloseAction", (int) Shutdown);
    }
    else if(ui.laptopClosedBatteryBlank->isChecked())
    {
        batSchemeConfig.lidCloseAction = Lock;
        cg.writeEntry("batSchemeConfig.lidCloseAction", (int) Lock);
    }
    if(ui.laptopClosedACNone->isChecked())
    {
        acSchemeConfig.lidCloseAction = None;
        cg.writeEntry("acSchemeConfig.lidCloseAction", (int) None);
    }
    else if(ui.laptopClosedACHibernate->isChecked())
    {
        acSchemeConfig.lidCloseAction = S2Disk;
        cg.writeEntry("acSchemeConfig.lidCloseAction", (int) S2Disk);
    }
    else if(ui.laptopClosedACSuspend->isChecked())
    {
        acSchemeConfig.lidCloseAction = S2Ram;
        cg.writeEntry("acSchemeConfig.lidCloseAction", (int) S2Ram);
    }
    else if(ui.laptopClosedACShutdown->isChecked())
    {
        acSchemeConfig.lidCloseAction = Shutdown;
        cg.writeEntry("acSchemeConfig.lidCloseAction", (int) Shutdown);
    }
    else if(ui.laptopClosedACBlank->isChecked())
    {
        acSchemeConfig.lidCloseAction = Lock;
        cg.writeEntry("acSchemeConfig.lidCloseAction", (int) Lock);
    }

    if (m_showBatteryString != ui.showBatteryStringCheckBox->isChecked()) {
        m_showBatteryString = !m_showBatteryString;
        cg.writeEntry("showPowersaveString", m_showBatteryString);
        showLabel(m_showBatteryString);
    }

    if (m_showMultipleBatteries != ui.showMultipleBatteriesCheckBox->isChecked()) {
        m_showMultipleBatteries = !m_showMultipleBatteries;
        cg.writeEntry("showMultipleBatteries", m_showMultipleBatteries);
        kDebug() << "Show multiple battery changed: " << m_showMultipleBatteries;
    }

    if (ui.styleGroup->selected() != m_batteryStyle) {
        QString svgFile = QString();
        if (ui.styleGroup->selected() == OxygenBattery) {
            svgFile = "widgets/battery-oxygen";
        } else {
            svgFile = "widgets/battery";
        }
        if (m_acadapter_plugged) {
            showAcAdapter(false);
        }
        showBattery(false);
        m_batteryStyle = ui.styleGroup->selected();
        delete m_theme;
        m_theme = new Plasma::Svg(this);
        m_theme->setImagePath(svgFile);
        kDebug() << "Changing theme to " << svgFile;
        cg.writeEntry("style", m_batteryStyle);
        m_theme->resize(contentsRect().size());
        if (m_acadapter_plugged) {
            showAcAdapter(true);
        }
        showBattery(true);
    }
    //Reset state
    acAdapterStateChanged(Solid::Control::PowerManager::acAdapterState());
}

void Powersave::dataUpdated(const QString& source, const Plasma::DataEngine::Data &data)
{
    if (source.startsWith(I18N_NOOP("Battery"))) {
        m_batteries_data[source] = data;
    } else if (source == I18N_NOOP("AC Adapter")) {
        m_acadapter_plugged = data[I18N_NOOP("Plugged in")].toBool();
        showAcAdapter(m_acadapter_plugged);
    } else {
        kDebug() << "Applet::Dunno what to do with " << source;
    }
    update();
}

void Powersave::createConfigurationInterface(KConfigDialog *parent)
{
    using namespace Solid::Control::PowerManager;
    QWidget * widget = new QWidget();
    ui.setupUi(widget);
    parent->setButtons( KDialog::Ok | KDialog::Cancel );
    parent->addPage(widget, parent->windowTitle(), icon());
    connect(parent, SIGNAL(applyClicked()), this, SLOT(configAccepted()));
    connect(parent, SIGNAL(okClicked()), this, SLOT(configAccepted()));
    ui.styleGroup->setSelected(m_batteryStyle);
    ui.showBatteryStringCheckBox->setChecked(m_showBatteryString ? Qt::Checked : Qt::Unchecked);
    ui.showMultipleBatteriesCheckBox->setChecked(m_showMultipleBatteries ? Qt::Checked : Qt::Unchecked);

    connect(ui.PoweredBrightnessSlider, SIGNAL(sliderMoved(int)), this, SLOT(configBrightnessChanged(int)));
    connect(ui.BatteryBrightnessSlider, SIGNAL(sliderMoved(int)), this, SLOT(configBrightnessChanged(int)));
    connect(this, SIGNAL(applyClicked()), this, SLOT(saveSettings()));
    connect(this, SIGNAL(okClicked()), this, SLOT(saveSettings()));
    ui.PoweredIdleCombo->addItem("Do nothing", 0);
    ui.PoweredIdleCombo->addItem("Shutdown", Shutdown);
    ui.BatteryCriticalCombo->addItem("Do nothing", 0);
    ui.BatteryCriticalCombo->addItem("Shutdown", Shutdown);
    ui.BatteryIdleCombo->addItem("Do nothing", 0);
    ui.BatteryIdleCombo->addItem("Shutdown", Shutdown);
    SuspendMethods methods = supportedSuspendMethods();
    if(methods | ToDisk)
    {
        ui.PoweredIdleCombo->addItem("Suspend to Disk", S2Disk);
        ui.BatteryCriticalCombo->addItem("Suspend to Disk", S2Disk);
        ui.BatteryIdleCombo->addItem("Suspend to Disk", S2Disk);
    }
    if(methods | ToRam)
    {
        ui.PoweredIdleCombo->addItem("Suspend to Ram", S2Ram);
        ui.BatteryCriticalCombo->addItem("Suspend to Ram", S2Ram);
        ui.BatteryIdleCombo->addItem("Suspend to Ram", S2Ram);
    }
    if(methods | Standby)
    {
        ui.PoweredIdleCombo->addItem("Standby", Powersave::Standby);
        ui.BatteryCriticalCombo->addItem("Standby", Powersave::Standby);
        ui.BatteryIdleCombo->addItem("Standby", Powersave::Standby);
    }
    CpuFreqPolicies policies = supportedCpuFreqPolicies();
    if(policies | Performance)
    {
        ui.PoweredFreqCombo->addItem("Performance", (int) Performance);
        ui.BatteryFreqCombo->addItem("Performance", (int) Performance);
    }
    if(policies | OnDemand)
    {
        ui.PoweredFreqCombo->addItem("Dynamic (aggressive)", (int) OnDemand);
        ui.BatteryFreqCombo->addItem("Dynamic (aggressive)", (int) OnDemand);
    }
    if(policies | Conservative)
    {
        ui.PoweredFreqCombo->addItem("Dynamic (less agressive)", (int) Conservative);
        ui.BatteryFreqCombo->addItem("Dynamic (less agressive)", (int) Conservative);
    }
    if(policies | Solid::Control::PowerManager::Powersave)
    {
        ui.PoweredFreqCombo->addItem("Powersave", (int) Solid::Control::PowerManager::Powersave);
        ui.BatteryFreqCombo->addItem("Powersave", (int) Solid::Control::PowerManager::Powersave);
    }
    readConfigToUi();
    configOpen = true;
}

void Powersave::mousePressEvent ( QGraphicsSceneMouseEvent * event )
{
    switch(event->button())
    {
        case Qt::LeftButton:
            mainMenu->popup(QPoint(event->screenPos()));
            break;
        default:
            break;
    }
    Applet::mousePressEvent(event);
}

void Powersave::wheelEvent ( QGraphicsSceneWheelEvent * event )
{
    currentBrightness += event->delta()/20;
    if(currentBrightness > 100)
    {
        currentBrightness = 100;
    }
    if(currentBrightness < 0)
    {
        currentBrightness = 0;
    }

    actionBrightnessSlider->setValue(currentBrightness);
    Solid::Control::PowerManager::setBrightness(currentBrightness);
}

void Powersave::configAccepted()
{
    configOpen = false;
    writeConfigFromUi();
    //reconnect sources
    disconnectSources();
    connectSources();

    emit configNeedsSaving();
}

void Powersave::readColors()
{
    m_textColor = Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor);
    m_boxColor = Plasma::Theme::defaultTheme()->color(Plasma::Theme::BackgroundColor);
    m_boxColor.setAlpha(m_boxAlpha);
}

void Powersave::hoverEnterEvent(QGraphicsSceneHoverEvent *event)
{
    showLabel(true);
    //showAcAdapter(false); // to test the animation without constant plugging
    //showPowersave(false); // to test the animation without constant plugging
    m_isHovered = true;
    Applet::hoverEnterEvent(event);
}

void Powersave::hoverLeaveEvent(QGraphicsSceneHoverEvent *event)
{
    if (!m_showBatteryString) {
        showLabel(false);
    }
    //showAcAdapter(true); // to test the animation without constant plugging
    //showPowersave(true); // to test the animation without constant plugging
    //m_isHovered = false;
    Applet::hoverLeaveEvent(event);
}

Powersave::~Powersave()
{
    /* These objects do not have a parent and hence i think it is necessary to explicitly delete them */
    delete mainMenu;
    delete m_grabber;
}

void Powersave::showLabel(bool show)
{
    if (m_fadeIn == show) {
        return;
    }
    m_fadeIn = show;
    const int FadeInDuration = 150;

    if (m_animId != -1) {
        Plasma::Animator::self()->stopCustomAnimation(m_animId);
    }
    m_animId = Plasma::Animator::self()->customAnimation(40 / (1000 / FadeInDuration), FadeInDuration,
                                                      Plasma::Animator::EaseOutCurve, this,
                                                      "animationUpdate");
}

QRectF Powersave::scaleRectF(const qreal progress, QRectF rect) {
    if (progress == 1) {
        return rect;
    }
    // Scale
    qreal w = rect.width()*progress;
    qreal h = rect.width()*progress;

    // Position centered
    rect.setX((rect.width() - w)/2);
    rect.setY((rect.height() - h)/2);

    rect.setWidth(w);
    rect.setHeight(h);

    return rect;
}

void Powersave::showAcAdapter(bool show)
{
    if (m_acFadeIn == show) {
        return;
    }
    m_acFadeIn = show;
    const int FadeInDuration = 300;
    // As long as the animation is running, we fake it's still plugged in so it gets
    // painted in paintInterface()
    m_acadapter_plugged = true;

    if (m_acAnimId != -1) {
        Plasma::Animator::self()->stopCustomAnimation(m_acAnimId);
    }
    m_acAnimId = Plasma::Animator::self()->customAnimation(40 / (1000 / FadeInDuration), FadeInDuration,
                                                      Plasma::Animator::EaseOutCurve, this,
                                                      "acAnimationUpdate");
}

void Powersave::showBattery(bool show)
{
    if (m_batteryFadeIn == show) {
        return;
    }
    m_batteryFadeIn = show;
    const int FadeInDuration = 300;

    if (m_batteryAnimId != -1) {
        Plasma::Animator::self()->stopCustomAnimation(m_batteryAnimId);
    }
    m_batteryAnimId = Plasma::Animator::self()->customAnimation(40 / (1000 / FadeInDuration), FadeInDuration,
                                                      Plasma::Animator::EaseOutCurve, this,
                                                      "batteryAnimationUpdate");
}

void Powersave::animationUpdate(qreal progress)
{
    if (progress == 1) {
        m_animId = -1;
    }
    if (!m_fadeIn) {
        qreal new_alpha = m_fadeIn ? progress : 1 - progress;
        m_alpha = qMin(new_alpha, m_alpha);
    } else {
        m_alpha = m_fadeIn ? progress : 1 - progress;
    }
    m_alpha = qMax(qreal(0.0), m_alpha);
    update();
}

void Powersave::acAnimationUpdate(qreal progress)
{
    if (progress == 1) {
        m_acAnimId = -1;
    }
    m_acAlpha = m_acFadeIn ? progress : 1 - progress;
    // During the fadeout animation, we had set it to true (and lie)
    // now the animation has ended, we _really_ set it to not show the adapter
    if (!m_acFadeIn && (progress == 1)) {
        m_acadapter_plugged = false;
    }
    update();
}

void Powersave::batteryAnimationUpdate(qreal progress)
{
    if (progress == 1) {
        m_batteryAnimId = -1;
    }
    m_batteryAlpha = m_batteryFadeIn ? progress : 1 - progress;
    update();
}

void Powersave::configBrightnessChanged(int brightness)
{
    Solid::Control::PowerManager::setBrightness(brightness);
}

void Powersave::paintLabel(QPainter *p, const QRect &contentsRect, const QString& labelText)
{
    // Store font size, we want to restore it shortly
    int original_font_size = m_font.pointSize();

    // Fonts smaller than smallestReadableFont don't make sense.
    m_font.setPointSize(qMax(KGlobalSettings::smallestReadableFont().pointSize(), m_font.pointSize()));
    QFontMetrics fm(m_font);
    qreal text_width = fm.width(labelText);

    // Longer texts get smaller fonts
    if (labelText.length() > 4) {
        if (original_font_size/1.5 < KGlobalSettings::smallestReadableFont().pointSize()) {
            m_font.setPointSize((KGlobalSettings::smallestReadableFont().pointSize()));
        } else {
            m_font.setPointSizeF(original_font_size/1.5);
        }
        fm = QFontMetrics(m_font);
        text_width = (fm.width(labelText) * 1.2);
    } else {
        // Smaller texts get a wider box
        text_width = (text_width * 1.4);
    }
    if (formFactor() == Plasma::Horizontal ||
        formFactor() == Plasma::Vertical) {
        m_font = KGlobalSettings::smallestReadableFont();
        m_font.setWeight(QFont::Bold);
        fm = QFontMetrics(m_font);
        text_width = (fm.width(labelText)+8);
    }
    p->setFont(m_font);

    // Let's find a good position for painting the background
    m_textRect = QRectF(qMax(qreal(0.0), contentsRect.left() + (contentsRect.width() - text_width) / 2),
                            contentsRect.top() + ((contentsRect.height() - (int)fm.height()) / 2 * 0.9),
                            qMin(contentsRect.width(), (int)text_width),
                            fm.height() * 1.2 );

    // Poor man's highlighting
    m_boxColor.setAlphaF(m_alpha);
    p->setPen(m_boxColor);
    m_boxColor.setAlphaF(m_alpha*0.5);
    p->setBrush(m_boxColor);

    // Find sensible proportions for the rounded corners
    float round_prop = m_textRect.width() / m_textRect.height();

    // Tweak the rounding edge a bit with the proportions of the textbox
    qreal round_radius = 35.0;
    p->drawRoundedRect(m_textRect, round_radius / round_prop, round_radius, Qt::RelativeSize);

    m_textColor.setAlphaF(m_alpha);
    p->setPen(m_textColor);
    p->drawText(m_textRect, Qt::AlignCenter, labelText);

    // Reset font and box
    m_font.setPointSize(original_font_size);
    m_boxColor.setAlpha(m_boxAlpha);
}

void Powersave::paintBattery(QPainter *p, const QRect &contentsRect, const int batteryPercent, const bool plugState)
{
    QString fill_element = QString();
    if (plugState && m_theme->hasElement("Battery")) {
        m_theme->paint(p, scaleRectF(m_batteryAlpha, contentsRect), "Battery");

        if (m_batteryStyle == OxygenBattery) {
            if (batteryPercent > 95) {
                fill_element = "Fill100";
            } else if (batteryPercent > 80) {
                fill_element = "Fill80";
            } else if (batteryPercent > 50) {
                fill_element = "Fill60";
            } else if (batteryPercent > 20) {
                fill_element = "Fill40";
            } else if (batteryPercent > 10) {
                fill_element = "Fill20";
            } // Don't show a fillbar below 11% charged
        } else { // OxyenStyle
            if (batteryPercent > 95) {
                fill_element = "Fill100";
            } else if (batteryPercent > 90) {
                fill_element = "Fill90";
            } else if (batteryPercent > 80) {
                fill_element = "Fill80";
            } else if (batteryPercent > 70) {
                fill_element = "Fill70";
            } else if (batteryPercent > 55) {
                fill_element = "Fill60";
            } else if (batteryPercent > 40) {
                fill_element = "Fill50";
            } else if (batteryPercent > 30) {
                fill_element = "Fill40";
            } else if (batteryPercent > 20) {
                fill_element = "Fill30";
            } else if (batteryPercent > 10) {
                fill_element = "Fill20";
            } else if (batteryPercent >= 5) {
                fill_element = "Fill10";
            } // Lower than 5%? Show no fillbar.
        }
    }
    //kDebug() << "plugState:" << plugState;

    // Now let's find out which fillstate to show
    if (plugState && !fill_element.isEmpty()) {
        if (m_theme->hasElement(fill_element)) {
            m_theme->paint(p, scaleRectF(m_batteryAlpha, contentsRect), fill_element);
        } else {
            kDebug() << fill_element << " does not exist in svg";
        }
    }

    if (m_acadapter_plugged) {
        //QRectF ac_rect = QRectF(contentsRect.topLeft(), QSizeF(contentsRect.width()*m_acAlpha, contentsRect.height()*m_acAlpha));
        m_theme->paint(p, scaleRectF(m_acAlpha, contentsRect), "AcAdapter");
    }

    // For small FormFactors, we're drawing a shadow
    if (formFactor() == Plasma::Vertical ||
        formFactor() == Plasma::Horizontal) {
        if (plugState) {
            m_theme->paint(p, contentsRect, "Shadow");
        }
    }
    if (plugState && m_theme->hasElement("Overlay")) {
        m_theme->paint(p, scaleRectF(m_batteryAlpha, contentsRect), "Overlay");
    }
}

void Powersave::paintInterface(QPainter *p, const QStyleOptionGraphicsItem *option, const QRect &contentsRect)
{
    Q_UNUSED( option );

    p->setRenderHint(QPainter::SmoothPixmapTransform);
    p->setRenderHint(QPainter::Antialiasing);

    if (m_numOfBattery == 0) {
        QRectF ac_contentsRect(contentsRect.topLeft(), QSizeF(qMax(qreal(0.0), contentsRect.width() * m_acAlpha), qMax(qreal(0.0), contentsRect.height() * m_acAlpha)));
        if (m_acadapter_plugged) {
            m_theme->paint(p, ac_contentsRect, "AcAdapter");
        }
        // Show that there's no battery
        paintLabel(p, contentsRect, I18N_NOOP("n/a"));
        return;
    }

    if (m_showMultipleBatteries) {
        // paint each battery with own charge level
        int battery_num = 0;
        int width = contentsRect.width()/m_numOfBattery;
        QHashIterator<QString, QHash<QString, QVariant > > battery_data(m_batteries_data);
        while (battery_data.hasNext()) {
            battery_data.next();
            QRect corect = QRect(contentsRect.left()+battery_num*width,
                                 contentsRect.top(),
                                 width, contentsRect.height());

            // paint battery with appropriate charge level
            paintBattery(p, corect, battery_data.value()[I18N_NOOP("Percent")].toInt(), battery_data.value()[I18N_NOOP("Plugged in")].toBool());

            if (m_showBatteryString || m_isHovered) {
                // Show the charge percentage with a box on top of the battery
                QString batteryLabel;
                if (battery_data.value()[I18N_NOOP("Plugged in")].toBool()) {
                    batteryLabel = battery_data.value()[I18N_NOOP("Percent")].toString();
                    batteryLabel.append("%");
                } else {
                    batteryLabel = I18N_NOOP("n/a");
                }
                paintLabel(p, corect, batteryLabel);
            }
            ++battery_num;
        }
    } else {
        // paint only one battery and show cumulative charge level
        int battery_num = 0;
        int battery_charge = 0;
        bool has_battery = false;
        QHashIterator<QString, QHash<QString, QVariant > > battery_data(m_batteries_data);
        while (battery_data.hasNext()) {
            battery_data.next();
            if (battery_data.value()[I18N_NOOP("Plugged in")].toBool()) {
                battery_charge += battery_data.value()[I18N_NOOP("Percent")].toInt();
                has_battery = true;
                ++battery_num;
            }
        }
        if (battery_num > 0) {
            battery_charge = battery_charge / battery_num;
        }
        // paint battery with appropriate charge level
        paintBattery(p, contentsRect,  battery_charge, has_battery);
        if (m_showBatteryString || m_isHovered) {
            // Show the charge percentage with a box on top of the battery
            QString batteryLabel;
            if(has_battery) {
                batteryLabel = QString::number(battery_charge);
                batteryLabel.append("%");
            } else {
                batteryLabel = I18N_NOOP("n/a");
            }
            paintLabel(p, contentsRect, batteryLabel);
        }
    }
}

void Powersave::connectSources() {
    const QStringList& battery_sources = dataEngine("powermanagement")->query(I18N_NOOP("Battery"))[I18N_NOOP("sources")].toStringList();

    foreach (const QString &battery_source, battery_sources) {
        dataEngine("powermanagement")->connectSource(battery_source, this);
    }

    dataEngine("powermanagement")->connectSource(I18N_NOOP("AC Adapter"), this);
}

void Powersave::disconnectSources()
{
    const QStringList& battery_sources = dataEngine("powermanagement")->query(I18N_NOOP("Battery"))[I18N_NOOP("sources")].toStringList();

    foreach (const QString &battery_source ,battery_sources) {
        dataEngine("powermanagement")->disconnectSource(battery_source, this);
    }

    dataEngine("powermanagement")->disconnectSource(I18N_NOOP("AC Adapter"), this);
}

void Powersave::cpuPolicyClicked()
{
    using namespace Solid::Control::PowerManager;
    actionCpuPolicyPerformance->setChecked(false);
    actionCpuPolicyOnDemand->setChecked(false);
    actionCpuPolicyConservative->setChecked(false);
    actionCpuPolicyPowersave->setChecked(false);
    bool ret = true;
    ret = setCpuFreqPolicy((CpuFreqPolicy)(sender()->property("CPUPolicy").toInt()));

    if(!ret)
    {
  //      KPassivePopup::message(i18n("WARNING"), i18n("Failed to switch CPU frequency policy"), //FIXME - ret value doesnt seem to be true?
  //                         KIcon("dialog-warning").pixmap(20,20), systray, 15000);
    }

    Solid::Control::PowerManager::CpuFreqPolicy currentPolicy = Solid::Control::PowerManager::cpuFreqPolicy();
    if(currentPolicy == Solid::Control::PowerManager::Performance)
    {
        actionCpuPolicyPerformance->setChecked(true);
    }
    else if(currentPolicy == Solid::Control::PowerManager::OnDemand)
    {
        actionCpuPolicyOnDemand->setChecked(true);
    }
    else if(currentPolicy == Solid::Control::PowerManager::Conservative)
    {
        actionCpuPolicyConservative->setChecked(true);
    }
    else if(currentPolicy == Solid::Control::PowerManager::Powersave)
    {
        actionCpuPolicyPowersave->setChecked(true);
    }
}

void Powersave::acAdapterStateChanged(int state)
{
    using namespace Solid::Control::PowerManager;
    struct PowerSchemeConfig *scheme = &batSchemeConfig;
    
    if(state == Plugged)
    {
        onAc = true;
        scheme = &acSchemeConfig;
    }
    else if (state == Unplugged)
    {
        onAc = false;
        scheme = &batSchemeConfig;
    }
    currentBrightness = scheme->brightness;
    setBrightness(scheme->brightness);
    setCpuFreqPolicy(scheme->cpuPolicy);

    actionBrightnessSlider->setValue(currentBrightness);
}

void Powersave::batteryStateChanged(int state)
{
    using namespace Solid::Control::PowerManager;
    switch(state)
    {
        case Critical:
            KPassivePopup::message(KPassivePopup::Boxed, "WARNING", "Battery is at critical level",
                           KIcon("dialog-warning").pixmap(20,20), dynamic_cast<QWidget *>(this), 15000);
            break;
        case Warning:
            KPassivePopup::message(KPassivePopup::Boxed, "WARNING", "Battery is at warning level",
                           KIcon("dialog-warning").pixmap(20,20), dynamic_cast<QWidget *>(this), 15000);
            break;
        case Low:
            KPassivePopup::message(KPassivePopup::Boxed, "WARNING", "Battery is at low level",
                           KIcon("dialog-warning").pixmap(20,20), dynamic_cast<QWidget *>(this), 15000);
            break;
    }
    if(state == Critical && !onAc)
    {
        switch(batLowAction)
        {
            case Shutdown:
                shutdown();
                break;
            case S2Disk:
                suspend(ToDisk);
                break;
            case S2Ram:
                suspend(ToRam);
                break;
            case Standby:
                suspend(Solid::Control::PowerManager::Standby);
                break;
            default:
                break;
        }
    }
}

void Powersave::buttonPressed(int but)
{
    using namespace Solid::Control::PowerManager;
    if(but == LidClose)
    {
        struct PowerSchemeConfig *scheme;
        if(onAc)
        {
            scheme = &acSchemeConfig;
        }
        else
        {
            scheme = &batSchemeConfig;
        }

        switch(scheme->lidCloseAction)
        {
            case Shutdown:
                shutdown();
                break;
            case S2Disk:
                suspend(ToDisk);
                break;
            case S2Ram:
                suspend(ToRam);
                break;
            case Standby:
                suspend(Solid::Control::PowerManager::Standby);
                break;
            case Lock:
                lockScreen();
                break;
            default:
                break;
        }
    } 
}

void Powersave::decreaseBrightness()
{
    currentBrightness -= 10;
    if(currentBrightness < 0)
    {
        currentBrightness = 0;
    }
    actionBrightnessSlider->setValue(currentBrightness);
    Solid::Control::PowerManager::setBrightness(currentBrightness);
}

void Powersave::increaseBrightness()
{
    currentBrightness += 10;
    if(currentBrightness > 100)
    {
        currentBrightness = 100;
    }
    actionBrightnessSlider->setValue(currentBrightness);
    Solid::Control::PowerManager::setBrightness(currentBrightness);
}

bool Powersave::eventFilter( QObject * object, QEvent * event )
{
    if((object == m_grabber)
        && (event->type() == QEvent::MouseMove || event->type() == QEvent::KeyPress))
    {
        detectedActivity();
    }
    
    return false;
}

void Powersave::detectedActivity()
{
    m_grabber->releaseMouse();
    m_grabber->releaseKeyboard();
    m_grabber->hide();
    
    //Reset the pollTimer;
    pollTimer->setInterval(0);
}

void Powersave::waitForActivity()
{
    m_grabber->show();
    m_grabber->grabMouse();
    m_grabber->grabKeyboard();
}

///HACK yucky
#include <QX11Info>
#include <X11/extensions/scrnsaver.h>

void Powersave::poll()
{
    using namespace Solid::Control::PowerManager;
    /* ///The way that it should be done (i think)
    ------------------------------------------------------------
    int idle = m_screenSaverIface->GetSessionIdleTime();
    ------------------------------------------------------------
    */
    ///The way that works
    //----------------------------------------------------------
    XScreenSaverInfo* mitInfo = 0;
    mitInfo = XScreenSaverAllocInfo ();
    XScreenSaverQueryInfo (QX11Info::display(), DefaultRootWindow (QX11Info::display()), mitInfo);
    int idle = mitInfo->idle/1000;
    //----------------------------------------------------------

    struct PowerSchemeConfig *scheme;
    if(onAc)
        scheme = &acSchemeConfig;
    else
        scheme = &batSchemeConfig;

    if(idle >= scheme->idleTime*60)
    {
        switch(scheme->idleAction)
        {
            case Shutdown:
                shutdown();
                break;
            case S2Disk:
                suspendToDisk();
                break;
            case S2Ram:
                suspendToRam();
                break;
            case Powersave::Standby:
                standby();
                break;
            default:
                break;
        }
    }
    else if(scheme->offDisplayWhenIdle && (idle >= (scheme->displayIdleTime*60)))
    {
        //FIXME Turn off monitor. How do i do this?
        //For the time being i know that the command "xset dpms force off"
        //turns off the monitor so i'll just use it
        QProcess::execute("xset dpms force off");
        pollTimer->setInterval((scheme->idleTime*60000) - (idle * 1000));
    }
    else if(dimOnIdle && (idle >= (scheme->displayIdleTime*60 *3/4))) //threefourths time - written this way for readability
    {
        setBrightness(0);
        pollTimer->setInterval((scheme->displayIdleTime*60000) - (idle * 1000));
    }
    else if(dimOnIdle && (idle >= (scheme->displayIdleTime*60 * 1/2))) //half time - written this way for readability
    {
        setBrightness(currentBrightness/2);
        waitForActivity();
        pollTimer->setInterval((scheme->displayIdleTime*60000 * 3/4) - (idle * 1000));
    }
    else
    {
        setBrightness(currentBrightness);
        pollTimer->setInterval((scheme->displayIdleTime*60000 * 1/2) - (idle * 1000));
    }
}

void Powersave::shutdown()
{
    displayManager->shutdown(KWorkSpace::ShutdownTypeHalt, KWorkSpace::ShutdownModeTryNow);
}

void Powersave::suspendToDisk()
{
    if(configLockScreen)
    {
        lockScreen();
    }
    KJob * job = Solid::Control::PowerManager::suspend(Solid::Control::PowerManager::ToDisk);
    connect(job, SIGNAL(result(KJob *)), this, SLOT(suspendJobResult(KJob *)));
    job->start();
}

void Powersave::suspendToRam()
{
    if(configLockScreen)
    {
        lockScreen();
    }
    KJob * job = Solid::Control::PowerManager::suspend(Solid::Control::PowerManager::ToRam);
    connect(job, SIGNAL(result(KJob *)), this, SLOT(suspendJobResult(KJob *)));
    job->start();
}

void Powersave::standby()
{
    if(configLockScreen)
    {
        lockScreen();
    }
    KJob * job = Solid::Control::PowerManager::suspend(Solid::Control::PowerManager::Standby);
    connect(job, SIGNAL(result(KJob *)), this, SLOT(suspendJobResult(KJob *)));
    job->start();
}

void Powersave::suspendJobResult(KJob * job)
{
    if(job->error())
    {
        KPassivePopup::message(KPassivePopup::Boxed, "WARNING", job->errorString(),
                           KIcon("dialog-warning").pixmap(20,20), dynamic_cast<QWidget *>(this), 15000);
    }
    m_screenSaverIface->SimulateUserActivity(); //prevent infinite suspension loops
}

void Powersave::lockScreen()
{
    m_screenSaverIface->Lock();
}

#include "powersave.moc"
