/***************************************************************************
 *   Copyright (C) 2007-2008 by Sebastian Kuegler <sebas@kde.org>          *
 *   Copyright (C) 2008 by Thomas Gillespie <tomjamesgillespie@gmail.com>  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#ifndef POWERSAVE_H
#define POWERSAVE_H

#include <QLabel>
#include <QGraphicsSceneHoverEvent>
#include <QPair>
#include <QMap>

#include <KAction>
#include <kworkspace/kdisplaymanager.h>
#include <KJob>
#include <KMenu>

#include <plasma/applet.h>
#include <plasma/animator.h>
#include <plasma/dataengine.h>

#include <solid/control/powermanager.h>

#include "ui_powersaveConfig.h"
#include "screensaver_interface.h"

namespace Plasma
{
    class Svg;
}

class Powersave : public Plasma::Applet
{
    Q_OBJECT
    public:
        Powersave(QObject *parent, const QVariantList &args);
        ~Powersave();

        void init();
        void paintInterface(QPainter *painter, const QStyleOptionGraphicsItem *option,
                            const QRect &contents);
        void setPath(const QString&);
        QSizeF contentSizeHint() const;
        Qt::Orientations expandingDirections() const;

        void constraintsEvent(Plasma::Constraints constraints);

    public slots:
        void acAdapterStateChanged(int);
        void batteryStateChanged(int);
        void buttonPressed(int);
        void cpuPolicyClicked();
        void dataUpdated(const QString &name, const Plasma::DataEngine::Data &data);
        void decreaseBrightness();
        void increaseBrightness();
        void poll();
        void shutdown();
        void suspendJobResult(KJob * job);
        void suspendToDisk();
        void suspendToRam();
        void standby();

    protected Q_SLOTS:
        virtual void hoverEnterEvent(QGraphicsSceneHoverEvent *event);
        virtual void hoverLeaveEvent(QGraphicsSceneHoverEvent *event);
        void configAccepted();
        void readColors();

    protected:
        void createConfigurationInterface(KConfigDialog *parent);
        void mousePressEvent ( QGraphicsSceneMouseEvent * event );
        void wheelEvent ( QGraphicsSceneWheelEvent * event );
        bool eventFilter( QObject * object, QEvent * event );
        void detectedActivity();
        void waitForActivity();
    private slots:
        void animationUpdate(qreal progress);
        void acAnimationUpdate(qreal progress);
        void batteryAnimationUpdate(qreal progress);
        void configBrightnessChanged(int);

    private:
        Q_ENUMS( m_batteryStyle )
        enum ClockStyle {
            // Keep the order of styles the same order as the items in the configdialog!
            OxygenBattery, ClassicBattery
        };
        enum IdleAction {
            Shutdown, S2Disk, S2Ram, Standby, Lock, None
        };
        void buildMenu();
        void connectSources();
        void disconnectSources();
        void readConfigToUi();
        void writeConfigFromUi();
        int m_batteryStyle;
        /* Paint battery with proper charge level */
        void paintBattery(QPainter *p, const QRect &contentsRect, const int batteryPercent, const bool plugState);
        /* Paint a label on top of the battery */
        void paintLabel(QPainter *p, const QRect &contentsRect, const QString& labelText);
        /* Fade in/out the label above the battery. */
        void showLabel(bool show);
        /* Scale in/out Battery. */
        void showBattery(bool show);
        /* Scale in/out Ac Adapter. */
        void showAcAdapter(bool show);
        /* Scale in a QRectF */
        QRectF scaleRectF(qreal progress, QRectF rect);
        /* Enable the screensaver */
        void lockScreen();
        /* Show multiple batteries with individual icons and charge info? */
        bool m_showMultipleBatteries;
        /* Should the battery charge information be shown on top? */
        bool m_showBatteryString;
        /* Should the screen lock on suspend? */
        bool configLockScreen;
        /* Should the screen dim when the display is idle? */
        bool dimOnIdle;
        struct PowerSchemeConfig
        {
            /* Brightness level for the power scheme */
            int brightness;
            /* Should the display turn off when idle for the power scheme*/
            bool offDisplayWhenIdle;
            /* Display poweroff idle time for the power scheme */
            int displayIdleTime;
            /* Idle timeout for the power scheme */
            int idleTime;
            /* Action to do on idle for the power scheme */
            IdleAction idleAction;
            /* Cpu policy for power scheme */
            Solid::Control::PowerManager::CpuFreqPolicy cpuPolicy;
            /* Action to do when lid is closed */
            IdleAction lidCloseAction;
        }acSchemeConfig, batSchemeConfig;
        /* Action to do when battery is low */
        IdleAction batLowAction;
        /* Dbus interface to the screensaver and idle time functions */
        OrgFreedesktopScreenSaverInterface * m_screenSaverIface;

        QSizeF m_size;
        int m_pixelSize;
        Plasma::Svg* m_theme;
        bool m_acadapter_plugged;
        KDisplayManager * displayManager;
        Solid::Control::PowerManager::Notifier * eventNotifier;

        // Configuration dialog
        Ui::powersaveConfig ui;

        int m_animId;
        qreal m_alpha;
        bool m_fadeIn;

        int m_acAnimId;
        qreal m_acAlpha;
        bool m_acFadeIn;

        int m_batteryAnimId;
        qreal m_batteryAlpha;
        bool m_batteryFadeIn;

        // Internal data
        QList<QVariant> batterylist, acadapterlist;
        QHash<QString, QHash<QString, QVariant> > m_batteries_data;
        QFont m_font;
        bool m_isHovered;
        QColor m_boxColor;
        QColor m_textColor;
        QRectF m_textRect;
        int m_boxAlpha;
        int m_boxHoverAlpha;
        int m_numOfBattery;
        bool onAc;
        bool configOpen;
        int currentBrightness;
        QTimer * pollTimer;
        /* This QWidget is used to detect user activity by grabbing the keyboard and mouse */
        QWidget *m_grabber;

        ///Menus
        KMenu * mainMenu;
        QAction * actionSuspendToDisk;
        QAction * actionSuspendToRam;
        QAction * actionStandby;
        QAction * actionCpuPolicyPerformance;
        QAction * actionCpuPolicyOnDemand;
        QAction * actionCpuPolicyConservative;
        QAction * actionCpuPolicyPowersave;
        QSlider * actionBrightnessSlider;
};

K_EXPORT_PLASMA_APPLET(powersave, Powersave)

#endif
